.audio_popup_play p, .audio_embedded_play p {
    display: inline;
}

.audio_popup_play a span, .audio_embedded_play span {
    display: none;
}

.coverImageContainer {
    float: left;
    margin: 0 1em 0.5em 0em;
}
#summary_wrapper {
    clear: both;
    background-color: #dfdfdf;
}
#objmetadata_pbwidgets_wrapper{
    width: 65%;
    border-right: 1px solid #fff;
    background-color: #efefef;
    position: relative;
}
#objmetadata_pbwidgets_wrapper.general-view {
    background-color: white;
    width: 100%;
}
#audio_metadata {
    float: right;
    width: 33%;
    padding-top: 1em;
    font-size: 90%;
}
#audio_metadata div {
    line-height: 1.6em;
}
#audio_metadata span {
    width: 45%
}
#playback_widgets {
    float: right;
    padding: 0.5em 1em 1em 1em;
}
#object_metadata {
    float: left;
    padding: 0.5em 1em 1em 1em;
}    
#play_icon {
    display: block;
    width: 16px; height: 16px;
    background: transparent url(../++resource++audioimages/play_icon.png) no-repeat left top;
}
#download_icon {
    display: block;
    width: 16px; height: 16px;
    background: transparent url(../++resource++audioimages/download_icon.png) no-repeat left top;
}
#popup_play_icon {
    display: block;
    width: 16px; height: 16px;
    background: transparent url(../++resource++audioimages/popup_play_icon.png) no-repeat left top;
}

.widget_icon {
    margin-bottom: 8px;
}

.image-absent {
    display: none;
}

.media-player {
    display: block;
    padding-bottom: 8px;
}

.media-player.media-absent {
    display: none;
}
.media-player.player-not-available{
    display: none;
}

.ogg-player {
    padding-bottom: 0px;
}

.bottom-content-info DT,
.bottom-content-info DD {
    float: left;
    margin: 0;
    padding: 0;
    line-height: 1.5em;
    font-size: 0.8em;
}

.bottom-content-info DT {
    clear: left;
    width: 10em;
    color: #666;
}

.bottom-content-info DD {
    padding-right: 0.5em;
}

.media-item {
    border: solid 1px #cacacb;
    padding: 0.75em;
    margin-top: 1.5em;
}

.media-item.even {
    background-color: transparent;
}

.media-item.odd {
    background-color: #f2f5f7;
}

.media-item .item-left {
    float: left;
    margin: 0;
    padding: 0;
    border-right: 1em solid transparent;
}

.media-item .item-content {
    padding-left: 1em;
    margin-left: 131px;
}

.media-item .item-left {
    padding: 0;
    margin: 0;
}

.media-item .item-left DL {
    padding: 0;
    margin: 0;
}

.media-item .item-left DT.album-art {
    display: none;
}

.media-item .item-left DD.album-art {
    border: solid 1px #cacacb;
    margin: 0 0 1em 0;
    width: 131px;
}

.media-item .item-left DD.album-art IMG {
    border: 1px solid transparent;
    width: 128px;
}

.media-item .item-left .no-album-art {
    margin: 1em auto;
    display: block;
    text-align: center;
}

.media-item .item-left DT,
.media-item .item-left DD {
    color: #9a9a99;
    padding: 0;
    margin: 0 0 0.2em 0;
    line-height: 1.1;
    font-size: 90%;
}

.media-item .item-left DT {
    float: left;
    padding-right: 0.5em;
}

.media-item .item-description {
    line-height: 1.2em;
    color: #9a9a99;
    font-size: 1.2em;
    margin-top: 0.5em;
}

.media-item .item-content DT {
    line-height: 1.4em;
    padding: 0 0.5em 0 0;
    margin: 0;
    float: left;
    clear: left;
    width: 6em;
}

.media-item .item-content DD {
    line-height: 1.4em;
    margin: 0;
    padding: 0 0.5em 0 0;
    float: left;
}

.media-item DT.tag {
    background: url(../++resource++audioimages/tags.gif) transparent no-repeat;
    padding-left: 20px;
}

.media-item DT.commenting {
    background: url(../++resource++audioimages/comments.gif) transparent no-repeat;
    padding-left: 20px;
}

.media-item DT.rating {
    background: url(../++resource++audioimages/ratings.gif) transparent no-repeat;
    padding-left: 20px;
}

.media-item H2 {
    border: 0;
    font-weight: bold;
    display: block;
}

.media-item .author-byline {
    color: #9a9a99;
    font-size: 90%;
}

.media-item .item-content DD {
    display: inline;
}

.media-item H2 .relevance {
    display: block;
    float: right;
    font-size: 75%;
    color: #9a9a99;
    font-weight: normal;
    margin-top: -1.5em;
}


.field #form\.title {
    width: 100%;
    font-size: 150%;
}

.field #form\.url {
    width: 100%;
}

.field #form\.tags {
    width: 100%;
}
